#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "visioExporter.h"


visioExporter::visioExporter(char *what_file, nodes *what_nodes) {
  filename = strdup(what_file);
  my_nodes = what_nodes;
}


visioExporter::~visioExporter() {
  free(filename);
}


void visioExporter::do_export() {
  FILE *outfile = fopen(filename, "w");
  write_headers(outfile);
  write_nodes(outfile);
  write_edges(outfile);
  fclose(outfile);
}


void visioExporter::write_headers(FILE *outfile) {
  const char *headers = "; visio csv\nTemplate,\"Block Diagram.vst\"\n\n";
  fwrite(headers, strlen(headers), 1, outfile);
}


void visioExporter::write_nodes(FILE *outfile) {
  smallest_x = 100000;
  smallest_y = 100000;
  node *temp_node;
  for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++) {
    temp_node = my_nodes->get_node(temp);
    if (temp_node->x_pos < smallest_x)
      smallest_x = (int) temp_node->x_pos;
    if (temp_node->y_pos < smallest_y)
      smallest_y = (int) temp_node->y_pos;
  }
  for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++) {
    temp_node = my_nodes->get_node(temp);
    write_node(temp_node, outfile);
  }
}


void visioExporter::write_node(node *this_node, FILE *outfile) {
   const char *MASTER_NAME = "";
   char *line = new char[200];
   sprintf(line, "Shape,\"%s\",\"%s\",\"%s\",%0.2f,%0.2f\n",
           this_node->s_name.c_str(), MASTER_NAME, this_node->s_name.c_str(),
           ((this_node->x_pos - smallest_x + 1) / 10.0),
           ((this_node->y_pos - smallest_y + 1) / 10.0));
  fwrite(line, strlen(line), 1, outfile);
  delete line;
}


void visioExporter::write_edges(FILE *outfile) {
  node *temp_node;
  node *temp_node2;
  for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++) {
    temp_node = my_nodes->get_node(temp);
    for (int temp2 = 0; temp2 < temp_node->number_of_connections; temp2++) {
       temp_node2 = temp_node->connecting_nodes[temp2];
       write_edge(temp_node, temp_node2, outfile);
    }
  }
}


void visioExporter::write_edge(node *from_node, node *to_node, FILE *outfile) {
   const char *MASTER_NAME = "Single arrowhead";
   char *line = new char[200];
   sprintf(line, "Link,\"\",\"%s\",\"\",\"%s\",\"%s\"\n",
           MASTER_NAME, from_node->s_name.c_str(), to_node->s_name.c_str());
  fwrite(line, strlen(line), 1, outfile);
  delete line;
}

